/*
    Freud - a standalone C++ port of the Emacs Psychiatrist
    Copyright (C) 2002  Markus Triska

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    You can contact me via triska@gmx.at
*/


#include <string>
#include <vector>

#include "wordtypes.h"

using namespace std;

bool WordTypeDeterminer::isNoun(const string &word)
{
	if (isType(word, "pronoun")) return true;

	if (word == "not" || isType(word, "verb") ||
		isType(word, "preposition") || isModifier(word)) return false;

	return true;
}

bool WordTypeDeterminer::isModifier(const string &word)
{
	return (isAdjective(word) || isAdverb(word) ||
		isType(word, "othermodifier"));
}

bool WordTypeDeterminer::isAdjective(const string &word)
{
	return (isType(word, "number") || isType(word, "article") ||
		isType(word, "colour") || isType(word, "size") ||
		isType(word, "possessivepronoun"));
}

bool WordTypeDeterminer::isAdverb(const string &word)
{
	if (word.size() > 2) {
		if (word[word.size() - 1] == 'l' &&
		    word[word.size() - 2] == 'y') return true;
	}
	return false;
}


extern WordTypeDeterminer wordtypes;

class WordCategorizer
{
public:
	WordCategorizer(const string &ptype) : type(ptype) { }
	void setType(const string &ptype) { type = ptype; }
	void operator() (const string &word) {
		// cout << "Categorizing: " << word << " as: " << type << endl;
		wordtypes.add(word, type);
	}
private:
	string type;
};

template<class T> T for_each_word(const string &, T);

void init_wordtypes()
{
	WordCategorizer wc("pronoun");
	string words = "i me mine myself we us ours ourselves ourself \
	you yours yourself yourselves \
	he him himself she hers herself \
	it that those this these things thing \
	they them themselves theirs \
	anybody everybody somebody \
	anyone everyone someone \
	anything something everything";
	for_each_word(words, wc);

	words = "how why who have has when where do is are";
	wc.setType("questionword");
	for_each_word(words, wc);

	words = "all always amusing any anyway associated awesome \
	bad beautiful best better but certain clear ever every fantastic \
	fun funny good great grody gross however if ignorant less linked \
	losing lusing many more much never nice obnoxious often poor pretty \
	real related rich similar some stupid super superb terrible terrific \
	too total tubular ugly very";
	wc.setType("othermodifier");
	for_each_word(words, wc);

	words = "about above after around as at before beneath behind beside \
	between by for from in inside into like near next of on onto over \
	same through thru to toward towards under underneath with without";
	wc.setType("preposition");
	for_each_word(words, wc);

	words = "one two three four five six seven eight nine ten eleven \
	twelve thirteen fourteen fifteen sixteen seventeen eighteen nineteen \
	twenty thirty forty fifty sixty seventy eighty ninety hundred \
	thousand million billion half quarter first second third fourth \
	fifth sixth seventh eighth ninth tenth";
	wc.setType("number");
	for_each_word(words, wc);

	words = "beige black blue brown crimson gray grey green orange \
	pink purple red tan tawny violet white yellow";
	wc.setType("colour");
	for_each_word(words, wc);

	words = "big large tall fat wide thick small petite short thin skinny";
	wc.setType("size");
	for_each_word(words, wc);

	words = "my your his her our their";
	wc.setType("possessivepronoun");
	for_each_word(words, wc);

	words = "abort aborted aborts ask asked asks am applied applies apply \
	are associate \
	be became become becomes becoming \
	been being believe believed believes \
	bit bite bites bore bored bores boring bought buy buys buying \
	call called calling calls came can caught catch come \
	contract contracted contracts control controlled controls \
	could croak croaks croaked cut cuts \
	dare dared define defines dial dialed dials did die died dies \
	dislike disliked \
	dislikes do does drank drink drinks drinking \
	drive drives driving drove dying \
	eat eating eats expand expanded expands \
	expect expected expects expel expels expelled \
	explain explained explains \
	fart farts feel feels felt fight fights find finds finding \
	forget forgets forgot fought found \
	fuck fucked fucking fucks \
	gave get gets getting give gives go goes going gone got gotten \
	had harm harms has hate hated hates have having \
	hear heard hears hearing help helped helping helps \
	hit hits hope hoped hopes hurt hurts \
	implies imply is \
	join joined joins jump jumped jumps \
	keep keeping keeps kept \
	kill killed killing kills kiss kissed kisses kissing \
	knew know knows \
	laid lay lays let lets lie lied lies like liked likes \
	liking listen listens \
	login look looked looking looks \
	lose losing lost \
	love loved loves loving \
	luse lusing lust lusts \
	made make makes making may mean means meant might \
	move moved moves moving must \
	need needed needs  \
	order ordered orders ought \
	paid pay pays pick picked picking picks  \
	placed placing prefer prefers put puts \
	ran rape raped rapes \
	read reading reads recall receive received receives \
	refer refered referred refers \
	relate related relates remember remembered remembers \
	romp romped romps run running runs \
	said sang sat saw say says \
	screw screwed screwing screws scrod see sees seem seemed \
	seems seen sell selling sells \
	send sendind sends sent shall shoot shot should \
	sing sings sit sits sitting sold studied study \
	take takes taking talk talked talking talks tell tells telling \
	think thinks \
	thought told took tooled touch touched touches touching \
	transfer transferred transfers transmit transmits transmitted \
	type types types typing \
	walk walked walking walks want wanted wants was watch \
	watched watching went were will wish would work worked works \
	write writes writing wrote use used uses using";
	wc.setType("verb");
	for_each_word(words, wc);

	wordtypes.add("the","article").add("a","article").add("an","article");
}

